@@ -1,7 +1,8 @@
 /*
  *  db.c
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
  *  Copyright (c) 2006 by David Kimpe <dnaku@frugalware.org>
@@ -85,9 +86,6 @@ static void _alpm_db_unregister(pmdb_t *db)
 		return;
 	}
 
-	_alpm_log(PM_LOG_DEBUG, "closing database '%s'\n", db->treename);
-	_alpm_db_close(db);
-
 	_alpm_log(PM_LOG_DEBUG, "unregistering database '%s'\n", db->treename);
 	_alpm_db_free(db);
 }
@@ -235,6 +233,7 @@ const char SYMEXPORT *alpm_db_get_url(const pmdb_t *db)
 	/* Sanity checks */
 	ASSERT(handle != NULL, return(NULL));
 	ASSERT(db != NULL, return(NULL));
+	ASSERT(db->servers != NULL, return(NULL));
 
 	url = (char*)db->servers->data;
 
@@ -263,7 +262,7 @@ pmpkg_t SYMEXPORT *alpm_db_get_pkg(pmdb_t *db, const char *name)
  * @param db pointer to the package database to get the package from
  * @return the list of packages on success, NULL on error
  */
-alpm_list_t SYMEXPORT *alpm_db_getpkgcache(pmdb_t *db)
+alpm_list_t SYMEXPORT *alpm_db_get_pkgcache(pmdb_t *db)
 {
 	ALPM_LOG_FUNC;
 
@@ -295,7 +294,7 @@ pmgrp_t SYMEXPORT *alpm_db_readgrp(pmdb_t *db, const char *name)
  * @param db pointer to the package database to get the group from
  * @return the list of groups on success, NULL on error
  */
-alpm_list_t SYMEXPORT *alpm_db_getgrpcache(pmdb_t *db)
+alpm_list_t SYMEXPORT *alpm_db_get_grpcache(pmdb_t *db)
 {
 	ALPM_LOG_FUNC;
 
@@ -400,10 +399,10 @@ alpm_list_t *_alpm_db_search(pmdb_t *db, const alpm_list_t *needles)
 			else if (desc && regexec(&reg, desc, 0, 0, 0) == 0) {
 				matched = desc;
 			}
-			/* check provides */
 			/* TODO: should we be doing this, and should we print something
 			 * differently when we do match it since it isn't currently printed? */
-			else {
+			if(!matched) {
+				/* check provides */
 				for(k = alpm_pkg_get_provides(pkg); k; k = k->next) {
 					if (regexec(&reg, k->data, 0, 0, 0) == 0) {
 						matched = k->data;
@@ -411,6 +410,15 @@ alpm_list_t *_alpm_db_search(pmdb_t *db, const alpm_list_t *needles)
 					}
 				}
 			}
+			if(!matched) {
+				/* check groups */
+				for(k = alpm_pkg_get_groups(pkg); k; k = k->next) {
+					if (regexec(&reg, k->data, 0, 0, 0) == 0) {
+						matched = k->data;
+						break;
+					}
+				}
+			}
 
 			if(matched != NULL) {
 				_alpm_log(PM_LOG_DEBUG, "    search target '%s' matched '%s'\n",
@@ -431,10 +439,8 @@ alpm_list_t *_alpm_db_search(pmdb_t *db, const alpm_list_t *needles)
 
 pmdb_t *_alpm_db_register_local(void)
 {
-	struct stat buf;
 	pmdb_t *db;
 	const char *dbpath;
-	char path[PATH_MAX];
 
 	ALPM_LOG_FUNC;
 
@@ -445,40 +451,23 @@ pmdb_t *_alpm_db_register_local(void)
 
 	_alpm_log(PM_LOG_DEBUG, "registering local database\n");
 
-	/* make sure the database directory exists */
 	dbpath = alpm_option_get_dbpath();
 	if(!dbpath) {
 		_alpm_log(PM_LOG_ERROR, _("database path is undefined\n"));
 			RET_ERR(PM_ERR_DB_OPEN, NULL);
 	}
-	snprintf(path, PATH_MAX, "%slocal", dbpath);
-	/* TODO this is rediculous, we try to do this even if we can't */
-	if(stat(path, &buf) != 0 || !S_ISDIR(buf.st_mode)) {
-		_alpm_log(PM_LOG_DEBUG, "database dir '%s' does not exist, creating it\n",
-				path);
-		if(_alpm_makepath(path) != 0) {
-			RET_ERR(PM_ERR_SYSTEM, NULL);
-		}
-	}
 
 	db = _alpm_db_new(dbpath, "local");
 	if(db == NULL) {
 		RET_ERR(PM_ERR_DB_CREATE, NULL);
 	}
 
-	_alpm_log(PM_LOG_DEBUG, "opening database '%s'\n", db->treename);
-	if(_alpm_db_open(db) == -1) {
-		_alpm_db_free(db);
-		RET_ERR(PM_ERR_DB_OPEN, NULL);
-	}
-
 	handle->db_local = db;
 	return(db);
 }
 
 pmdb_t *_alpm_db_register_sync(const char *treename)
 {
-	struct stat buf;
 	pmdb_t *db;
 	const char *dbpath;
 	char path[PATH_MAX];
@@ -496,25 +485,12 @@ pmdb_t *_alpm_db_register_sync(const char *treename)
 
 	_alpm_log(PM_LOG_DEBUG, "registering sync database '%s'\n", treename);
 
-	/* make sure the database directory exists */
 	dbpath = alpm_option_get_dbpath();
 	if(!dbpath) {
 		_alpm_log(PM_LOG_ERROR, _("database path is undefined\n"));
 			RET_ERR(PM_ERR_DB_OPEN, NULL);
 	}
 	/* all sync DBs now reside in the sync/ subdir of the dbpath */
-	snprintf(path, PATH_MAX, "%ssync/%s", dbpath, treename);
-	/* TODO this is rediculous, we try to do this even if we can't */
-	if(stat(path, &buf) != 0 || !S_ISDIR(buf.st_mode)) {
-		_alpm_log(PM_LOG_DEBUG, "database dir '%s' does not exist, creating it\n",
-				path);
-		if(_alpm_makepath(path) != 0) {
-			RET_ERR(PM_ERR_SYSTEM, NULL);
-		}
-	}
-
-	/* Ensure the db gets the real path. */
-	path[0] = '\0';
 	snprintf(path, PATH_MAX, "%ssync/", dbpath);
 
 	db = _alpm_db_new(path, treename);
@@ -522,12 +498,6 @@ pmdb_t *_alpm_db_register_sync(const char *treename)
 		RET_ERR(PM_ERR_DB_CREATE, NULL);
 	}
 
-	_alpm_log(PM_LOG_DEBUG, "opening database '%s'\n", db->treename);
-	if(_alpm_db_open(db) == -1) {
-		_alpm_db_free(db);
-		RET_ERR(PM_ERR_DB_OPEN, NULL);
-	}
-
 	handle->dbs_sync = alpm_list_add(handle->dbs_sync, db);
 	return(db);
 }
