@@ -124,6 +124,10 @@ static int download_internal(const char *url, const char *localpath,
 	destfile = get_destfile(localpath, filename);
 	tempfile = get_tempfile(localpath, filename);
 
+	if(mtimeold) {
+		fileurl->last_modified = mtimeold;
+	}
+
 	/* pass the raw filename for passing to the callback function */
 	_alpm_log(PM_LOG_DEBUG, "using '%s' for download progress\n", filename);
 
@@ -147,8 +151,8 @@ static int download_internal(const char *url, const char *localpath,
 	/* libfetch does not reset the error code */
 	fetchLastErrCode = 0;
 
-	/* 10s timeout - TODO make a config option */
-	fetchTimeout = 10000;
+	/* 10s timeout */
+	fetchTimeout = 10;
 
 	/* ignore any SIGPIPE signals- these may occur if our FTP socket dies or
 	 * something along those lines. Store the old signal handler first. */
@@ -157,7 +161,13 @@ static int download_internal(const char *url, const char *localpath,
 	sigaction(SIGPIPE, NULL, &old_action);
 	sigaction(SIGPIPE, &new_action, NULL);
 
-	dlf = fetchXGet(fileurl, &ust, (handle->nopassiveftp ? "" : "p"));
+	dlf = fetchXGet(fileurl, &ust, (handle->nopassiveftp ? "i" : "pi"));
+
+	if(fetchLastErrCode == FETCH_UNCHANGED) {
+		_alpm_log(PM_LOG_DEBUG, "mtimes are identical, skipping %s\n", filename);
+		ret = 1;
+		goto cleanup;
+	}
 
 	if(fetchLastErrCode != 0 || dlf == NULL) {
 		const char *host = _("disk");
@@ -173,12 +183,6 @@ static int download_internal(const char *url, const char *localpath,
 		_alpm_log(PM_LOG_DEBUG, "connected to %s successfully\n", fileurl->host);
 	}
 
-	if(ust.mtime && mtimeold && ust.mtime == mtimeold) {
-		_alpm_log(PM_LOG_DEBUG, "mtimes are identical, skipping %s\n", filename);
-		ret = 1;
-		goto cleanup;
-	}
-
 	if(ust.mtime && mtimenew) {
 		*mtimenew = ust.mtime;
 	}
