@@ -1,7 +1,8 @@
 /*
  *  handle.h
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,6 +41,7 @@ typedef struct _pmhandle_t {
 	alpm_cb_log logcb;      /* Log callback function */
 	alpm_cb_download dlcb;  /* Download callback function */
 	alpm_cb_totaldl totaldlcb;  /* Total download callback function */
+	alpm_cb_fetch fetchcb; /* Download file callback function */
 
 	/* filesystem paths */
 	char *root;              /* Root path, default '/' */
@@ -52,13 +54,11 @@ typedef struct _pmhandle_t {
 	alpm_list_t *noupgrade;   /* List of packages NOT to be upgraded */
 	alpm_list_t *noextract;   /* List of files NOT to extract */
 	alpm_list_t *ignorepkg;   /* List of packages to ignore */
-	alpm_list_t *holdpkg;     /* List of packages which 'hold' pacman */
 	alpm_list_t *ignoregrp;   /* List of groups to ignore */
 
 	/* options */
 	unsigned short usesyslog;    /* Use syslog instead of logfile? */ /* TODO move to frontend */
 	unsigned short nopassiveftp; /* Don't use PASV ftp connections */
-	char *xfercommand;        /* External download command */
 	unsigned short usedelta;     /* Download deltas if possible */
 } pmhandle_t;
 
