@@ -1,7 +1,8 @@
 /*
  *  sync.c
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
  *  Copyright (c) 2005, 2006 by Miklos Vajna <vmiklos@frugalware.org>
@@ -27,6 +28,7 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
+#include <stdint.h> /* intmax_t */
 #include <unistd.h>
 #include <time.h>
 #include <dirent.h>
@@ -47,130 +49,15 @@
 #include "dload.h"
 #include "delta.h"
 
-pmsyncpkg_t *_alpm_sync_new(pmpkgreason_t newreason, pmpkg_t *spkg, alpm_list_t *removes)
-{
-	pmsyncpkg_t *sync;
-
-	ALPM_LOG_FUNC;
-
-	CALLOC(sync, 1, sizeof(pmsyncpkg_t), RET_ERR(PM_ERR_MEMORY, NULL));
-
-	sync->newreason = newreason;
-	sync->pkg = spkg;
-	sync->removes = removes;
-
-	return(sync);
-}
-
-void _alpm_sync_free(pmsyncpkg_t *sync)
-{
-	ALPM_LOG_FUNC;
-
-	if(sync == NULL) {
-		return;
-	}
-
-	alpm_list_free(sync->removes);
-	sync->removes = NULL;
-	FREE(sync);
-}
-
-/* Find recommended replacements for packages during a sync.
- */
-static int find_replacements(pmtrans_t *trans, pmdb_t *db_local,
-		alpm_list_t *dbs_sync, alpm_list_t **syncpkgs)
-{
-	alpm_list_t *i, *j, *k; /* wow */
-
-	ALPM_LOG_FUNC;
-
-	if(syncpkgs == NULL) {
-		return(-1);
-	}
-
-	/* check for "recommended" package replacements */
-	_alpm_log(PM_LOG_DEBUG, "checking for package replacements\n");
-	for(i = dbs_sync; i; i = i->next) {
-		pmdb_t *db = i->data;
-
-		/* for each db, check each package's REPLACES list */
-		for(j = _alpm_db_get_pkgcache(db); j; j = j->next) {
-			pmpkg_t *spkg = j->data;
-
-			for(k = alpm_pkg_get_replaces(spkg); k; k = k->next) {
-				const char *replacement = k->data;
-
-				pmpkg_t *lpkg = _alpm_db_get_pkgfromcache(db_local, replacement);
-				if(!lpkg) {
-					continue;
-				}
-
-				_alpm_log(PM_LOG_DEBUG, "checking replacement '%s' for package '%s'\n",
-						replacement, spkg->name);
-				/* ignore if EITHER the local or replacement package are to be ignored */
-				if(_alpm_pkg_should_ignore(spkg) || _alpm_pkg_should_ignore(lpkg)) {
-					_alpm_log(PM_LOG_WARNING, _("%s-%s: ignoring package upgrade (to be replaced by %s-%s)\n"),
-										alpm_pkg_get_name(lpkg), alpm_pkg_get_version(lpkg),
-										alpm_pkg_get_name(spkg), alpm_pkg_get_version(spkg));
-				} else {
-					/* get confirmation for the replacement */
-					if(trans) {
-						int doreplace = 0;
-						QUESTION(trans, PM_TRANS_CONV_REPLACE_PKG, lpkg, spkg, db->treename, &doreplace);
-						if(!doreplace) {
-							continue;
-						}
-					}
-
-					/* if confirmed, add this to the 'final' list, designating 'lpkg' as
-					 * the package to replace.
-					 */
-					pmsyncpkg_t *sync;
-
-					/* check if spkg->name is already in the packages list. */
-					/* TODO: same package name doesn't mean same package */
-					sync = _alpm_sync_find(*syncpkgs, alpm_pkg_get_name(spkg));
-					if(sync) {
-						/* found it -- just append to the removes list */
-						sync->removes = alpm_list_add(sync->removes, lpkg);
-						/* check the to-be-replaced package's reason field */
-						if(lpkg->reason == PM_PKG_REASON_EXPLICIT) {
-							sync->newreason = PM_PKG_REASON_EXPLICIT;
-						}
-					} else {
-						/* none found -- enter pkg into the final sync list */
-						/* copy over reason */
-						sync = _alpm_sync_new(alpm_pkg_get_reason(lpkg), spkg, NULL);
-						if(sync == NULL) {
-							pm_errno = PM_ERR_MEMORY;
-							alpm_list_free_inner(*syncpkgs, (alpm_list_fn_free)_alpm_sync_free);
-							alpm_list_free(*syncpkgs);
-							*syncpkgs = NULL;
-							return(-1);
-						}
-						sync->removes = alpm_list_add(NULL, lpkg);
-						_alpm_log(PM_LOG_DEBUG, "adding package %s-%s to the transaction targets\n",
-									alpm_pkg_get_name(spkg), alpm_pkg_get_version(spkg));
-						*syncpkgs = alpm_list_add(*syncpkgs, sync);
-					}
-					_alpm_log(PM_LOG_DEBUG, "%s-%s elected for removal (to be replaced by %s-%s)\n",
-							alpm_pkg_get_name(lpkg), alpm_pkg_get_version(lpkg),
-							alpm_pkg_get_name(spkg), alpm_pkg_get_version(spkg));
-				}
-			}
-		}
-	}
-	return(0);
-}
-
 /** Check for new version of pkg in sync repos
  * (only the first occurrence is considered in sync)
  */
 pmpkg_t SYMEXPORT *alpm_sync_newversion(pmpkg_t *pkg, alpm_list_t *dbs_sync)
 {
+	ASSERT(pkg != NULL, return(NULL));
+
 	alpm_list_t *i;
 	pmpkg_t *spkg = NULL;
-	int cmp;
 
 	for(i = dbs_sync; !spkg && i; i = i->next) {
 		spkg = _alpm_db_get_pkgfromcache(i->data, alpm_pkg_get_name(pkg));
@@ -183,98 +70,120 @@ pmpkg_t SYMEXPORT *alpm_sync_newversion(pmpkg_t *pkg, alpm_list_t *dbs_sync)
 	}
 
 	/* compare versions and see if spkg is an upgrade */
-	cmp = _alpm_pkg_compare_versions(spkg, pkg);
-	if(cmp > 0) {
+	if(_alpm_pkg_compare_versions(spkg, pkg) > 0) {
 		_alpm_log(PM_LOG_DEBUG, "new version of '%s' found (%s => %s)\n",
 					alpm_pkg_get_name(pkg), alpm_pkg_get_version(pkg),
 					alpm_pkg_get_version(spkg));
 		return(spkg);
 	}
-	if (cmp < 0) {
-			pmdb_t *db = spkg->origin_data.db;
-			_alpm_log(PM_LOG_WARNING, _("%s: local (%s) is newer than %s (%s)\n"),
-					alpm_pkg_get_name(pkg), alpm_pkg_get_version(pkg),
-					alpm_db_get_name(db), alpm_pkg_get_version(spkg));
-	}
+	/* spkg is not an upgrade */
 	return(NULL);
 }
 
-/** Get a list of upgradable packages on the current system
- * Adds out of date packages to *list.
- * @arg list pointer to a list of pmsyncpkg_t.
- */
-int SYMEXPORT alpm_sync_sysupgrade(pmdb_t *db_local,
-		alpm_list_t *dbs_sync, alpm_list_t **syncpkgs)
+int _alpm_sync_sysupgrade(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync, int enable_downgrade)
 {
-	return(_alpm_sync_sysupgrade(NULL, db_local, dbs_sync, syncpkgs));
-}
-
-int _alpm_sync_sysupgrade(pmtrans_t *trans,
-		pmdb_t *db_local, alpm_list_t *dbs_sync, alpm_list_t **syncpkgs)
-{
-	alpm_list_t *i, *j, *replaced = NULL;
+	alpm_list_t *i, *j, *k;
 
 	ALPM_LOG_FUNC;
 
-	if(syncpkgs == NULL) {
-		return(-1);
-	}
-	/* check for "recommended" package replacements */
-	if(find_replacements(trans, db_local, dbs_sync, syncpkgs)) {
-		return(-1);
-	}
-
-	/* compute the to-be-replaced packages for efficiency */
-	for(i = *syncpkgs; i; i = i->next) {
-		pmsyncpkg_t *sync = i->data;
-		for(j = sync->removes; j; j = j->next) {
-			replaced = alpm_list_add(replaced, j->data);
-		}
-	}
-
-	/* for all not-replaced local package we check for upgrade */
 	_alpm_log(PM_LOG_DEBUG, "checking for package upgrades\n");
 	for(i = _alpm_db_get_pkgcache(db_local); i; i = i->next) {
-		pmpkg_t *local = i->data;
+		pmpkg_t *lpkg = i->data;
 
-		if(_alpm_pkg_find(replaced, alpm_pkg_get_name(local))) {
-			_alpm_log(PM_LOG_DEBUG, "'%s' is already elected for removal -- skipping\n",
-					alpm_pkg_get_name(local));
+		if(_alpm_pkg_find(trans->packages, lpkg->name)) {
+			_alpm_log(PM_LOG_DEBUG, "%s is already in the target list -- skipping\n", lpkg->name);
 			continue;
 		}
 
-		pmpkg_t *spkg = alpm_sync_newversion(local, dbs_sync);
-		if(spkg) {
-			/* we found a new version */
-			/* skip packages in IgnorePkg or in IgnoreGroup */
-			if(_alpm_pkg_should_ignore(spkg)) {
-				_alpm_log(PM_LOG_WARNING, _("%s: ignoring package upgrade (%s => %s)\n"),
-						alpm_pkg_get_name(local), alpm_pkg_get_version(local),
-						alpm_pkg_get_version(spkg));
-				continue;
-			}
+		/* Search for literal then replacers in each sync database.
+		 * If found, don't check other databases */
+		for(j = dbs_sync; j; j = j->next) {
+			pmdb_t *sdb = j->data;
+			/* Check sdb */
+			pmpkg_t *spkg = _alpm_db_get_pkgfromcache(sdb, lpkg->name);
+			if(spkg) { /* 1. literal was found in sdb */
+				int cmp = _alpm_pkg_compare_versions(spkg, lpkg);
+				if(cmp > 0) {
+					_alpm_log(PM_LOG_DEBUG, "new version of '%s' found (%s => %s)\n",
+								lpkg->name, lpkg->version, spkg->version);
+					/* check IgnorePkg/IgnoreGroup */
+					if(_alpm_pkg_should_ignore(spkg) || _alpm_pkg_should_ignore(lpkg)) {
+						_alpm_log(PM_LOG_WARNING, _("%s: ignoring package upgrade (%s => %s)\n"),
+										lpkg->name, lpkg->version, spkg->version);
+					} else {
+						_alpm_log(PM_LOG_DEBUG, "adding package %s-%s to the transaction targets\n",
+												spkg->name, spkg->version);
+						trans->packages = alpm_list_add(trans->packages, spkg);
+					}
+				} else if(cmp < 0) {
+					if(enable_downgrade) {
+						/* check IgnorePkg/IgnoreGroup */
+						if(_alpm_pkg_should_ignore(spkg) || _alpm_pkg_should_ignore(lpkg)) {
+							_alpm_log(PM_LOG_WARNING, _("%s: ignoring package downgrade (%s => %s)\n"),
+											lpkg->name, lpkg->version, spkg->version);
+						} else {
+							_alpm_log(PM_LOG_WARNING, _("%s: downgrading from version %s to version %s\n"),
+											lpkg->name, lpkg->version, spkg->version);
+							trans->packages = alpm_list_add(trans->packages, spkg);
+						}
+					} else {
+						_alpm_log(PM_LOG_WARNING, _("%s: local (%s) is newer than %s (%s)\n"),
+								lpkg->name, lpkg->version, sdb->treename, spkg->version);
+					}
+				}
+				break; /* jump to next local package */
+			} else { /* 2. search for replacers in sdb */
+				int found = 0;
+				for(k = _alpm_db_get_pkgcache(sdb); k; k = k->next) {
+					spkg = k->data;
+					if(alpm_list_find_str(alpm_pkg_get_replaces(spkg), lpkg->name)) {
+						found = 1;
+						/* check IgnorePkg/IgnoreGroup */
+						if(_alpm_pkg_should_ignore(spkg) || _alpm_pkg_should_ignore(lpkg)) {
+							_alpm_log(PM_LOG_WARNING, _("ignoring package replacement (%s-%s => %s-%s)\n"),
+										lpkg->name, lpkg->version, spkg->name, spkg->version);
+							continue;
+						}
 
-			/* add the upgrade package to our pmsyncpkg_t list */
-			if(_alpm_sync_find(*syncpkgs, alpm_pkg_get_name(spkg))) {
-				/* avoid duplicated targets */
-				continue;
+						int doreplace = 0;
+						QUESTION(trans, PM_TRANS_CONV_REPLACE_PKG, lpkg, spkg, sdb->treename, &doreplace);
+						if(!doreplace) {
+							continue;
+						}
+
+						/* If spkg is already in the target list, we append lpkg to spkg's removes list */
+						pmpkg_t *tpkg = _alpm_pkg_find(trans->packages, spkg->name);
+						if(tpkg) {
+							/* sanity check, multiple repos can contain spkg->name */
+							if(tpkg->origin_data.db != sdb) {
+								_alpm_log(PM_LOG_WARNING, _("cannot replace %s by %s\n"),
+													lpkg->name, spkg->name);
+								continue;
+							}
+							_alpm_log(PM_LOG_DEBUG, "appending %s to the removes list of %s\n",
+												lpkg->name, tpkg->name);
+							tpkg->removes = alpm_list_add(tpkg->removes, lpkg);
+							/* check the to-be-replaced package's reason field */
+							if(alpm_pkg_get_reason(lpkg) == PM_PKG_REASON_EXPLICIT) {
+								tpkg->reason = PM_PKG_REASON_EXPLICIT;
+							}
+						} else { /* add spkg to the target list */
+							/* copy over reason */
+							spkg->reason = alpm_pkg_get_reason(lpkg);
+							spkg->removes = alpm_list_add(NULL, lpkg);
+							_alpm_log(PM_LOG_DEBUG, "adding package %s-%s to the transaction targets\n",
+													spkg->name, spkg->version);
+							trans->packages = alpm_list_add(trans->packages, spkg);
+						}
+					}
+				}
+				if(found) {
+					break; /* jump to next local package */
+				}
 			}
-			/* we can set any reason here, it will be overridden by add_commit */
-			pmsyncpkg_t *sync = _alpm_sync_new(PM_PKG_REASON_EXPLICIT, spkg, NULL);
-			if(sync == NULL) {
-				alpm_list_free_inner(*syncpkgs, (alpm_list_fn_free)_alpm_sync_free);
-				alpm_list_free(*syncpkgs);
-				*syncpkgs = NULL;
-				alpm_list_free(replaced);
-				return(-1);
-			}
-			_alpm_log(PM_LOG_DEBUG, "adding package %s-%s to the transaction targets\n",
-							alpm_pkg_get_name(spkg), alpm_pkg_get_version(spkg));
-			*syncpkgs = alpm_list_add(*syncpkgs, sync);
 		}
 	}
 
-	alpm_list_free(replaced);
 	return(0);
 }
 
@@ -313,21 +222,22 @@ int _alpm_sync_addtarget(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sy
 			RET_ERR(PM_ERR_PKG_REPO_NOT_FOUND, -1);
 		}
 		dep = _alpm_splitdep(targ);
-		spkg = _alpm_resolvedep(dep, dbs, NULL, NULL);
+		spkg = _alpm_resolvedep(dep, dbs, NULL, 1);
 		_alpm_dep_free(dep);
 		alpm_list_free(dbs);
 	} else {
 		dep = _alpm_splitdep(targline);
-		spkg = _alpm_resolvedep(dep, dbs_sync, NULL, NULL);
+		spkg = _alpm_resolvedep(dep, dbs_sync, NULL, 1);
 		_alpm_dep_free(dep);
 	}
 	FREE(targline);
 
 	if(spkg == NULL) {
-		RET_ERR(PM_ERR_PKG_NOT_FOUND, -1);
+		/* pm_errno is set by _alpm_resolvedep */
+		return(-1);
 	}
 
-	if(_alpm_sync_find(trans->packages, alpm_pkg_get_name(spkg))) {
+	if(_alpm_pkg_find(trans->packages, alpm_pkg_get_name(spkg))) {
 		RET_ERR(PM_ERR_TRANS_DUP_TARGET, -1);
 	}
 
@@ -354,28 +264,14 @@ int _alpm_sync_addtarget(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sy
 	}
 
 	/* add the package to the transaction */
-	pmsyncpkg_t *sync = _alpm_sync_new(PM_PKG_REASON_EXPLICIT, spkg, NULL);
-	if(sync == NULL) {
-		return(-1);
-	}
+	spkg->reason = PM_PKG_REASON_EXPLICIT;
 	_alpm_log(PM_LOG_DEBUG, "adding package %s-%s to the transaction targets\n",
 						alpm_pkg_get_name(spkg), alpm_pkg_get_version(spkg));
-	trans->packages = alpm_list_add(trans->packages, sync);
+	trans->packages = alpm_list_add(trans->packages, spkg);
 
 	return(0);
 }
 
-/* Helper functions for alpm_list_remove
-*/
-static int syncpkg_cmp(const void *s1, const void *s2)
-{
-	const pmsyncpkg_t *sp1 = s1;
-	const pmsyncpkg_t *sp2 = s2;
-	pmpkg_t *p1 = alpm_sync_get_pkg(sp1);
-	pmpkg_t *p2 = alpm_sync_get_pkg(sp2);
-	return(strcmp(alpm_pkg_get_name(p1), alpm_pkg_get_name(p2)));
-}
-
 /** Compute the size of the files that will be downloaded to install a
  * package.
  * @param newpkg the new package to upgrade to
@@ -400,7 +296,6 @@ static int compute_download_size(pmpkg_t *newpkg)
 		dltsize = _alpm_shortest_delta_path(
 			alpm_pkg_get_deltas(newpkg),
 			alpm_pkg_get_filename(newpkg),
-			alpm_pkg_get_md5sum(newpkg),
 			&newpkg->delta_path);
 
 		if(newpkg->delta_path && (dltsize < pkgsize * MAX_DELTA_RATIO)) {
@@ -416,8 +311,8 @@ static int compute_download_size(pmpkg_t *newpkg)
 		size = alpm_pkg_get_size(newpkg);
 	}
 
-	_alpm_log(PM_LOG_DEBUG, "setting download size %lld for pkg %s\n",
-			(long long)size, alpm_pkg_get_name(newpkg));
+	_alpm_log(PM_LOG_DEBUG, "setting download size %jd for pkg %s\n",
+			(intmax_t)size, alpm_pkg_get_name(newpkg));
 
 	newpkg->download_size = size;
 	return(0);
@@ -426,7 +321,9 @@ static int compute_download_size(pmpkg_t *newpkg)
 int _alpm_sync_prepare(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync, alpm_list_t **data)
 {
 	alpm_list_t *deps = NULL;
-	alpm_list_t *list = NULL, *remove = NULL; /* allow checkdeps usage with trans->packages */
+	alpm_list_t *preferred = NULL;
+	alpm_list_t *unresolvable = NULL;
+	alpm_list_t *remove = NULL; /* allow checkdeps usage with trans->packages */
 	alpm_list_t *i, *j;
 	int ret = 0;
 
@@ -439,66 +336,83 @@ int _alpm_sync_prepare(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync
 		*data = NULL;
 	}
 
-	for(i = trans->packages; i; i = i->next) {
-		pmsyncpkg_t *sync = i->data;
-		list = alpm_list_add(list, sync->pkg);
-	}
-
 	if(!(trans->flags & PM_TRANS_FLAG_NODEPS)) {
-		/* store a pointer to the last original target so we can tell what was
-		 * pulled by resolvedeps */
-		alpm_list_t *pulled = alpm_list_last(list);
+		alpm_list_t *resolved = NULL; /* target list after resolvedeps */
+
+		/* Build up list by repeatedly resolving each transaction package */
 		/* Resolve targets dependencies */
 		EVENT(trans, PM_TRANS_EVT_RESOLVEDEPS_START, NULL, NULL);
 		_alpm_log(PM_LOG_DEBUG, "resolving target's dependencies\n");
 
-		/* build remove list for resolvedeps */
+		/* build remove list and preferred list for resolvedeps */
 		for(i = trans->packages; i; i = i->next) {
-			pmsyncpkg_t *sync = i->data;
-			for(j = sync->removes; j; j = j->next) {
+			pmpkg_t *spkg = i->data;
+			for(j = spkg->removes; j; j = j->next) {
 				remove = alpm_list_add(remove, j->data);
 			}
+			preferred = alpm_list_add(preferred, spkg);
 		}
 
-		if(_alpm_resolvedeps(db_local, dbs_sync, list, remove, data) == -1) {
-			/* pm_errno is set by resolvedeps */
-			ret = -1;
-			goto cleanup;
+		/* Resolve packages in the transaction one at a time, in addtion
+		   building up a list of packages which could not be resolved. */
+		for(i = trans->packages; i; i = i->next) {
+			pmpkg_t *pkg = i->data;
+			if(_alpm_resolvedeps(db_local, dbs_sync, pkg, preferred,
+						&resolved, remove, data) == -1) {
+				unresolvable = alpm_list_add(unresolvable, pkg);
+			}
+			/* Else, [resolved] now additionally contains [pkg] and all of its
+			   dependencies not already on the list */
 		}
+		alpm_list_free(preferred);
 
-		for(i = pulled->next; i; i = i->next) {
-			pmpkg_t *spkg = i->data;
-			pmsyncpkg_t *sync = _alpm_sync_new(PM_PKG_REASON_DEPEND, spkg, NULL);
-			if(sync == NULL) {
+		/* If there were unresolvable top-level packages, prompt the user to
+		   see if they'd like to ignore them rather than failing the sync */
+		if(unresolvable != NULL) {
+			unresolvable = alpm_list_remove_dupes(unresolvable);
+			int remove_unresolvable = 0;
+			QUESTION(handle->trans, PM_TRANS_CONV_REMOVE_PKGS, unresolvable,
+					NULL, NULL, &remove_unresolvable);
+			if (remove_unresolvable) {
+				/* User wants to remove the unresolvable packages from the
+				   transaction. The packages will be removed from the actual
+				   transaction when the transaction packages are replaced with a
+				   dependency-reordered list below */
+				pm_errno = 0; /* pm_errno was set by resolvedeps */
+				if(data) {
+					alpm_list_free_inner(*data, (alpm_list_fn_free)_alpm_depmiss_free);
+					alpm_list_free(*data);
+					*data = NULL;
+				}
+			} else {
+				/* pm_errno is set by resolvedeps */
+				alpm_list_free(resolved);
 				ret = -1;
 				goto cleanup;
 			}
-			trans->packages = alpm_list_add(trans->packages, sync);
-			_alpm_log(PM_LOG_DEBUG, "adding package %s-%s to the transaction targets\n",
-								alpm_pkg_get_name(spkg), alpm_pkg_get_version(spkg));
+		}
+
+		/* Unresolvable packages will be removed from the target list, so
+		   we free the transaction specific fields */
+		alpm_list_free_inner(unresolvable, (alpm_list_fn_free)_alpm_pkg_free_trans);
+
+		/* Set DEPEND reason for pulled packages */
+		for(i = resolved; i; i = i->next) {
+			pmpkg_t *pkg = i->data;
+			if(!_alpm_pkg_find(trans->packages, pkg->name)) {
+				pkg->reason = PM_PKG_REASON_DEPEND;
+			}
 		}
 
 		/* re-order w.r.t. dependencies */
-		alpm_list_t *sortlist = _alpm_sortbydeps(list, 0);
-		alpm_list_t *newpkgs = NULL;
-		for(i = sortlist; i; i = i->next) {
-			for(j = trans->packages; j; j = j->next) {
-				pmsyncpkg_t *s = j->data;
-				if(s->pkg == i->data) {
-					newpkgs = alpm_list_add(newpkgs, s);
-					break;
-				}
-			}
-		}
-		alpm_list_free(sortlist);
 		alpm_list_free(trans->packages);
-		trans->packages = newpkgs;
+		trans->packages = _alpm_sortbydeps(resolved, 0);
+		alpm_list_free(resolved);
 
 		EVENT(trans, PM_TRANS_EVT_RESOLVEDEPS_DONE, NULL, NULL);
 	}
 
-	/* We don't care about conflicts if we're just printing uris */
-	if(!(trans->flags & (PM_TRANS_FLAG_NOCONFLICTS | PM_TRANS_FLAG_PRINTURIS))) {
+	if(!(trans->flags & PM_TRANS_FLAG_NOCONFLICTS)) {
 		/* check for inter-conflicts and whatnot */
 		EVENT(trans, PM_TRANS_EVT_INTERCONFLICTS_START, NULL, NULL);
 
@@ -506,15 +420,15 @@ int _alpm_sync_prepare(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync
 
 		/* 1. check for conflicts in the target list */
 		_alpm_log(PM_LOG_DEBUG, "check targets vs targets\n");
-		deps = _alpm_innerconflicts(list);
+		deps = _alpm_innerconflicts(trans->packages);
 
 		for(i = deps; i; i = i->next) {
 			pmconflict_t *conflict = i->data;
-			pmsyncpkg_t *rsync, *sync, *sync1, *sync2;
+			pmpkg_t *rsync, *sync, *sync1, *sync2;
 
 			/* have we already removed one of the conflicting targets? */
-			sync1 = _alpm_sync_find(trans->packages, conflict->package1);
-			sync2 = _alpm_sync_find(trans->packages, conflict->package2);
+			sync1 = _alpm_pkg_find(trans->packages, conflict->package1);
+			sync2 = _alpm_pkg_find(trans->packages, conflict->package2);
 			if(!sync1 || !sync2) {
 				continue;
 			}
@@ -525,10 +439,10 @@ int _alpm_sync_prepare(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync
 			/* if sync1 provides sync2, we remove sync2 from the targets, and vice versa */
 			pmdepend_t *dep1 = _alpm_splitdep(conflict->package1);
 			pmdepend_t *dep2 = _alpm_splitdep(conflict->package2);
-			if(alpm_depcmp(sync1->pkg, dep2)) {
+			if(alpm_depcmp(sync1, dep2)) {
 				rsync = sync2;
 				sync = sync1;
-			} else if(alpm_depcmp(sync2->pkg, dep1)) {
+			} else if(alpm_depcmp(sync2, dep1)) {
 				rsync = sync1;
 				sync = sync2;
 			} else {
@@ -553,13 +467,9 @@ int _alpm_sync_prepare(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync
 			/* Prints warning */
 			_alpm_log(PM_LOG_WARNING,
 					_("removing '%s' from target list because it conflicts with '%s'\n"),
-					rsync->pkg->name, sync->pkg->name);
-			void *vpkg;
-			trans->packages = alpm_list_remove(trans->packages, rsync,
-					syncpkg_cmp, &vpkg);
-			pmsyncpkg_t *syncpkg = vpkg;
-			list = alpm_list_remove(list, syncpkg->pkg, _alpm_pkg_cmp, NULL);
-			_alpm_sync_free(syncpkg);
+					rsync->name, sync->name);
+			_alpm_pkg_free_trans(rsync); /* rsync is not transaction target anymore */
+			trans->packages = alpm_list_remove(trans->packages, rsync, _alpm_pkg_cmp, NULL);
 			continue;
 		}
 
@@ -569,7 +479,7 @@ int _alpm_sync_prepare(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync
 
 		/* 2. we check for target vs db conflicts (and resolve)*/
 		_alpm_log(PM_LOG_DEBUG, "check targets vs db and db vs targets\n");
-		deps = _alpm_outerconflicts(db_local, list);
+		deps = _alpm_outerconflicts(db_local, trans->packages);
 
 		for(i = deps; i; i = i->next) {
 			pmconflict_t *conflict = i->data;
@@ -578,8 +488,8 @@ int _alpm_sync_prepare(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync
 			   we ask the user */
 			int found = 0;
 			for(j = trans->packages; j && !found; j = j->next) {
-				pmsyncpkg_t *sync = j->data;
-				if(_alpm_pkg_find(sync->removes, conflict->package2)) {
+				pmpkg_t *spkg = j->data;
+				if(_alpm_pkg_find(spkg->removes, conflict->package2)) {
 					found = 1;
 				}
 			}
@@ -590,7 +500,7 @@ int _alpm_sync_prepare(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync
 			_alpm_log(PM_LOG_DEBUG, "package '%s' conflicts with '%s'\n",
 					conflict->package1, conflict->package2);
 
-			pmsyncpkg_t *sync = _alpm_sync_find(trans->packages, conflict->package1);
+			pmpkg_t *sync = _alpm_pkg_find(trans->packages, conflict->package1);
 			pmpkg_t *local = _alpm_db_get_pkgfromcache(db_local, conflict->package2);
 			int doremove = 0;
 			QUESTION(trans, PM_TRANS_CONV_CONFLICT_PKG, conflict->package1,
@@ -624,14 +534,14 @@ int _alpm_sync_prepare(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync
 		alpm_list_free(remove);
 		remove = NULL;
 		for(i = trans->packages; i; i = i->next) {
-			pmsyncpkg_t *sync = i->data;
-			for(j = sync->removes; j; j = j->next) {
+			pmpkg_t *spkg = i->data;
+			for(j = spkg->removes; j; j = j->next) {
 				remove = alpm_list_add(remove, j->data);
 			}
 		}
 
 		_alpm_log(PM_LOG_DEBUG, "checking dependencies\n");
-		deps = alpm_checkdeps(db_local, 1, remove, list);
+		deps = alpm_checkdeps(_alpm_db_get_pkgcache(db_local), 1, remove, trans->packages);
 		if(deps) {
 			pm_errno = PM_ERR_UNSATISFIED_DEPS;
 			ret = -1;
@@ -644,7 +554,7 @@ int _alpm_sync_prepare(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync
 			goto cleanup;
 		}
 	}
-	for(i = list; i; i = i->next) {
+	for(i = trans->packages; i; i = i->next) {
 		/* update download size field */
 		pmpkg_t *spkg = i->data;
 		if(compute_download_size(spkg) != 0) {
@@ -654,8 +564,8 @@ int _alpm_sync_prepare(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync
 	}
 
 cleanup:
-	alpm_list_free(list);
 	alpm_list_free(remove);
+	alpm_list_free(unresolvable);
 
 	return(ret);
 }
@@ -670,6 +580,12 @@ off_t SYMEXPORT alpm_pkg_download_size(pmpkg_t *newpkg)
 	return(newpkg->download_size);
 }
 
+static int endswith(const char *filename, const char *extension)
+{
+	const char *s = filename + strlen(filename) - strlen(extension);
+	return(strcmp(s, extension) == 0);
+}
+
 /** Applies delta files to create an upgraded package file.
  *
  * All intermediate files are deleted, leaving only the starting and
@@ -686,8 +602,7 @@ static int apply_deltas(pmtrans_t *trans)
 	const char *cachedir = _alpm_filecache_setup();
 
 	for(i = trans->packages; i; i = i->next) {
-		pmsyncpkg_t *sync = i->data;
-		pmpkg_t *spkg = sync->pkg;
+		pmpkg_t *spkg = i->data;
 		alpm_list_t *delta_path = spkg->delta_path;
 		alpm_list_t *dlts = NULL;
 
@@ -715,16 +630,15 @@ static int apply_deltas(pmtrans_t *trans)
 			CALLOC(to, len, sizeof(char), RET_ERR(PM_ERR_MEMORY, 1));
 			snprintf(to, len, "%s/%s", cachedir, d->to);
 
-			/* an example of the patch command: (using /cache for cachedir)
-			 * xdelta patch /path/to/pacman_3.0.0-1_to_3.0.1-1-i686.delta \
-			 *              /path/to/pacman-3.0.0-1-i686.pkg.tar.gz       \
-			 *              /cache/pacman-3.0.1-1-i686.pkg.tar.gz
-			 */
-
 			/* build the patch command */
-			snprintf(command, PATH_MAX, "xdelta patch %s %s %s", delta, from, to);
+			if(endswith(to, ".gz")) {
+				/* special handling for gzip : we disable timestamp with -n option */
+				snprintf(command, PATH_MAX, "xdelta3 -d -q -R -c -s %s %s | gzip -n > %s", from, delta, to);
+			} else {
+				snprintf(command, PATH_MAX, "xdelta3 -d -q -s %s %s %s", from, delta, to);
+			}
 
-			_alpm_log(PM_LOG_DEBUG, _("command: %s\n"), command);
+			_alpm_log(PM_LOG_DEBUG, "command: %s\n", command);
 
 			EVENT(trans, PM_TRANS_EVT_DELTA_PATCH_START, d->to, d->delta);
 
@@ -797,7 +711,7 @@ int _alpm_sync_commit(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t **data)
 {
 	alpm_list_t *i, *j, *files = NULL;
 	alpm_list_t *deltas = NULL;
-	pmtrans_t *tr = NULL;
+	pmtrans_t *tr_remove = NULL, *tr_upgrade = NULL;
 	int replaces = 0;
 	int errors = 0;
 	const char *cachedir = NULL;
@@ -817,8 +731,7 @@ int _alpm_sync_commit(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t **data)
 		off_t total_size = (off_t)0;
 		/* sum up the download size for each package and store total */
 		for(i = trans->packages; i; i = i->next) {
-			pmsyncpkg_t *sync = i->data;
-			pmpkg_t *spkg = sync->pkg;
+			pmpkg_t *spkg = i->data;
 			total_size += spkg->download_size;
 		}
 		handle->totaldlcb(total_size);
@@ -829,8 +742,7 @@ int _alpm_sync_commit(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t **data)
 		pmdb_t *current = i->data;
 
 		for(j = trans->packages; j; j = j->next) {
-			pmsyncpkg_t *sync = j->data;
-			pmpkg_t *spkg = sync->pkg;
+			pmpkg_t *spkg = j->data;
 			pmdb_t *dbs = spkg->origin_data.db;
 
 			if(current == dbs) {
@@ -838,34 +750,31 @@ int _alpm_sync_commit(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t **data)
 
 				fname = alpm_pkg_get_filename(spkg);
 				ASSERT(fname != NULL, RET_ERR(PM_ERR_PKG_INVALID_NAME, -1));
-				if(trans->flags & PM_TRANS_FLAG_PRINTURIS) {
-					EVENT(trans, PM_TRANS_EVT_PRINTURI, (char *)alpm_db_get_url(current),
-							(char *)fname);
-				} else {
-					if(spkg->download_size != 0) {
-						alpm_list_t *delta_path = spkg->delta_path;
-						if(delta_path) {
-							alpm_list_t *dlts = NULL;
+				alpm_list_t *delta_path = spkg->delta_path;
+				if(delta_path) {
+					/* using deltas */
+					alpm_list_t *dlts = NULL;
 
-							for(dlts = delta_path; dlts; dlts = dlts->next) {
-								pmdelta_t *d = dlts->data;
+					for(dlts = delta_path; dlts; dlts = dlts->next) {
+						pmdelta_t *d = dlts->data;
 
-								if(d->download_size != 0) {
-									/* add the delta filename to the download list if
-									 * it's not in the cache */
-									files = alpm_list_add(files, strdup(d->delta));
-								}
-
-								/* keep a list of the delta files for md5sums */
-								deltas = alpm_list_add(deltas, d);
-							}
-
-						} else {
-							/* not using deltas, so add the file to the download list */
-							files = alpm_list_add(files, strdup(fname));
+						if(d->download_size != 0) {
+							/* add the delta filename to the download list if needed */
+							files = alpm_list_add(files, strdup(d->delta));
 						}
+
+						/* keep a list of all the delta files for md5sums */
+						deltas = alpm_list_add(deltas, d);
+					}
+
+				} else {
+					/* not using deltas */
+					if(spkg->download_size != 0) {
+						/* add the filename to the download list if needed */
+						files = alpm_list_add(files, strdup(fname));
 					}
 				}
+
 			}
 		}
 
@@ -874,52 +783,48 @@ int _alpm_sync_commit(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t **data)
 			if(_alpm_download_files(files, current->servers, cachedir)) {
 				_alpm_log(PM_LOG_WARNING, _("failed to retrieve some files from %s\n"),
 						current->treename);
-				pm_errno = PM_ERR_RETRIEVE;
+				if(pm_errno == 0) {
+					pm_errno = PM_ERR_RETRIEVE;
+				}
 				goto error;
 			}
 			FREELIST(files);
 		}
 	}
-	if(trans->flags & PM_TRANS_FLAG_PRINTURIS) {
-		return(0);
-	}
 
 	/* clear out value to let callback know we are done */
 	if(handle->totaldlcb) {
 		handle->totaldlcb(0);
 	}
 
-	if(handle->usedelta) {
+	/* if we have deltas to work with */
+	if(handle->usedelta && deltas) {
 		int ret = 0;
+		errors = 0;
+		/* Check integrity of deltas */
+		EVENT(trans, PM_TRANS_EVT_DELTA_INTEGRITY_START, NULL, NULL);
 
-		/* only output if there are deltas to work with */
-		if(deltas) {
-			errors = 0;
-			/* Check integrity of deltas */
-			EVENT(trans, PM_TRANS_EVT_DELTA_INTEGRITY_START, NULL, NULL);
+		for(i = deltas; i; i = i->next) {
+			pmdelta_t *d = alpm_list_getdata(i);
+			const char *filename = alpm_delta_get_filename(d);
+			const char *md5sum = alpm_delta_get_md5sum(d);
 
-			for(i = deltas; i; i = i->next) {
-				pmdelta_t *d = alpm_list_getdata(i);
-				const char *filename = alpm_delta_get_filename(d);
-				const char *md5sum = alpm_delta_get_md5sum(d);
-
-				if(test_md5sum(trans, filename, md5sum) != 0) {
-					errors++;
-					*data = alpm_list_add(*data, strdup(filename));
-				}
+			if(test_md5sum(trans, filename, md5sum) != 0) {
+				errors++;
+				*data = alpm_list_add(*data, strdup(filename));
 			}
-			if(errors) {
-				pm_errno = PM_ERR_DLT_INVALID;
-				goto error;
-			}
-			EVENT(trans, PM_TRANS_EVT_DELTA_INTEGRITY_DONE, NULL, NULL);
-
-			/* Use the deltas to generate the packages */
-			EVENT(trans, PM_TRANS_EVT_DELTA_PATCHES_START, NULL, NULL);
-			ret = apply_deltas(trans);
-			EVENT(trans, PM_TRANS_EVT_DELTA_PATCHES_DONE, NULL, NULL);
-
 		}
+		if(errors) {
+			pm_errno = PM_ERR_DLT_INVALID;
+			goto error;
+		}
+		EVENT(trans, PM_TRANS_EVT_DELTA_INTEGRITY_DONE, NULL, NULL);
+
+		/* Use the deltas to generate the packages */
+		EVENT(trans, PM_TRANS_EVT_DELTA_PATCHES_START, NULL, NULL);
+		ret = apply_deltas(trans);
+		EVENT(trans, PM_TRANS_EVT_DELTA_PATCHES_DONE, NULL, NULL);
+
 		if(ret) {
 			pm_errno = PM_ERR_DLT_PATCHFAILED;
 			goto error;
@@ -931,8 +836,7 @@ int _alpm_sync_commit(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t **data)
 
 	errors = 0;
 	for(i = trans->packages; i; i = i->next) {
-		pmsyncpkg_t *sync = i->data;
-		pmpkg_t *spkg = sync->pkg;
+		pmpkg_t *spkg = i->data;
 		const char *filename = alpm_pkg_get_filename(spkg);
 		const char *md5sum = alpm_pkg_get_md5sum(spkg);
 
@@ -950,62 +854,44 @@ int _alpm_sync_commit(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t **data)
 		return(0);
 	}
 
-	/* remove conflicting and to-be-replaced packages */
 	trans->state = STATE_COMMITING;
-	tr = _alpm_trans_new();
-	if(tr == NULL) {
+
+	/* Create remove and upgrade transactions */
+	tr_remove = _alpm_trans_new();
+	if(tr_remove == NULL) {
 		_alpm_log(PM_LOG_ERROR, _("could not create removal transaction\n"));
 		goto error;
 	}
-
-	if(_alpm_trans_init(tr, PM_TRANS_TYPE_REMOVE, PM_TRANS_FLAG_NODEPS, NULL, NULL, NULL) == -1) {
-		_alpm_log(PM_LOG_ERROR, _("could not initialize the removal transaction\n"));
+	tr_upgrade = _alpm_trans_new();
+	if(tr_upgrade == NULL) {
+		_alpm_log(PM_LOG_ERROR, _("could not create transaction\n"));
 		goto error;
 	}
 
+	if(_alpm_trans_init(tr_remove, PM_TRANS_TYPE_REMOVE, PM_TRANS_FLAG_NODEPS, NULL, NULL, NULL) == -1) {
+		_alpm_log(PM_LOG_ERROR, _("could not initialize the removal transaction\n"));
+		goto error;
+	}
+	if(_alpm_trans_init(tr_upgrade, PM_TRANS_TYPE_UPGRADE, trans->flags, trans->cb_event, trans->cb_conv, trans->cb_progress) == -1) {
+		_alpm_log(PM_LOG_ERROR, _("could not initialize transaction\n"));
+		goto error;
+	}
+
+	/* adding targets */
 	for(i = trans->packages; i; i = i->next) {
-		pmsyncpkg_t *sync = i->data;
+		pmpkg_t *spkg = i->data;
 		alpm_list_t *j;
-		for(j = sync->removes; j; j = j->next) {
+		/* remove transaction */
+		for(j = spkg->removes; j; j = j->next) {
 			pmpkg_t *pkg = j->data;
-			if(!_alpm_pkg_find(tr->packages, pkg->name)) {
-				if(_alpm_trans_addtarget(tr, pkg->name) == -1) {
+			if(!_alpm_pkg_find(tr_remove->packages, pkg->name)) {
+				if(_alpm_trans_addtarget(tr_remove, pkg->name) == -1) {
 					goto error;
 				}
 				replaces++;
 			}
 		}
-	}
-	if(replaces) {
-		_alpm_log(PM_LOG_DEBUG, "removing conflicting and to-be-replaced packages\n");
-		if(_alpm_trans_prepare(tr, data) == -1) {
-			_alpm_log(PM_LOG_ERROR, _("could not prepare removal transaction\n"));
-			goto error;
-		}
-		/* we want the frontend to be aware of commit details */
-		tr->cb_event = trans->cb_event;
-		if(_alpm_trans_commit(tr, NULL) == -1) {
-			_alpm_log(PM_LOG_ERROR, _("could not commit removal transaction\n"));
-			goto error;
-		}
-	}
-	_alpm_trans_free(tr);
-	tr = NULL;
-
-	/* install targets */
-	_alpm_log(PM_LOG_DEBUG, "installing packages\n");
-	tr = _alpm_trans_new();
-	if(tr == NULL) {
-		_alpm_log(PM_LOG_ERROR, _("could not create transaction\n"));
-		goto error;
-	}
-	if(_alpm_trans_init(tr, PM_TRANS_TYPE_UPGRADE, trans->flags | PM_TRANS_FLAG_NODEPS, trans->cb_event, trans->cb_conv, trans->cb_progress) == -1) {
-		_alpm_log(PM_LOG_ERROR, _("could not initialize transaction\n"));
-		goto error;
-	}
-	for(i = trans->packages; i; i = i->next) {
-		pmsyncpkg_t *sync = i->data;
-		pmpkg_t *spkg = sync->pkg;
+		/* upgrade transaction */
 		const char *fname;
 		char *fpath;
 
@@ -1016,7 +902,7 @@ int _alpm_sync_commit(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t **data)
 		/* Loop through the cache dirs until we find a matching file */
 		fpath = _alpm_filecache_find(fname);
 
-		if(_alpm_trans_addtarget(tr, fpath) == -1) {
+		if(_alpm_trans_addtarget(tr_upgrade, fpath) == -1) {
 			FREE(fpath);
 			goto error;
 		}
@@ -1024,15 +910,51 @@ int _alpm_sync_commit(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t **data)
 
 		/* using alpm_list_last() is ok because addtarget() adds the new target at the
 		 * end of the tr->packages list */
-		spkg = alpm_list_last(tr->packages)->data;
-		spkg->reason = sync->newreason;
+		pmpkg_t *ipkg = alpm_list_last(tr_upgrade->packages)->data;
+		ipkg->reason = spkg->reason;
 	}
-	if(_alpm_trans_prepare(tr, data) == -1) {
-		_alpm_log(PM_LOG_ERROR, _("could not prepare transaction\n"));
-		/* pm_errno is set by trans_prepare */
-		goto error;
+
+	/* fileconflict check */
+	if(!(trans->flags & PM_TRANS_FLAG_FORCE)) {
+		EVENT(trans, PM_TRANS_EVT_FILECONFLICTS_START, NULL, NULL);
+
+		_alpm_log(PM_LOG_DEBUG, "looking for file conflicts\n");
+		alpm_list_t *conflict = _alpm_db_find_fileconflicts(db_local, tr_upgrade,
+								    tr_upgrade->packages, tr_remove->packages);
+		if(conflict) {
+			pm_errno = PM_ERR_FILE_CONFLICTS;
+			if(data) {
+				*data = conflict;
+			} else {
+				alpm_list_free_inner(conflict, (alpm_list_fn_free)_alpm_fileconflict_free);
+				alpm_list_free(conflict);
+			}
+			goto error;
+		}
+
+		EVENT(trans, PM_TRANS_EVT_FILECONFLICTS_DONE, NULL, NULL);
 	}
-	if(_alpm_trans_commit(tr, NULL) == -1) {
+
+	/* remove conflicting and to-be-replaced packages */
+	if(replaces) {
+		_alpm_log(PM_LOG_DEBUG, "removing conflicting and to-be-replaced packages\n");
+		if(_alpm_trans_prepare(tr_remove, data) == -1) {
+			_alpm_log(PM_LOG_ERROR, _("could not prepare removal transaction\n"));
+			goto error;
+		}
+		/* we want the frontend to be aware of commit details */
+		tr_remove->cb_event = trans->cb_event;
+		tr_remove->cb_progress = trans->cb_progress;
+		if(_alpm_trans_commit(tr_remove, NULL) == -1) {
+			_alpm_log(PM_LOG_ERROR, _("could not commit removal transaction\n"));
+			goto error;
+		}
+	}
+
+	/* install targets */
+	_alpm_log(PM_LOG_DEBUG, "installing packages\n");
+	/* add_prepare is not needed */
+	if(_alpm_trans_commit(tr_upgrade, NULL) == -1) {
 		_alpm_log(PM_LOG_ERROR, _("could not commit transaction\n"));
 		goto error;
 	}
@@ -1041,44 +963,9 @@ int _alpm_sync_commit(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t **data)
 error:
 	FREELIST(files);
 	alpm_list_free(deltas);
-	deltas = NULL;
-	_alpm_trans_free(tr);
-	tr = NULL;
+	_alpm_trans_free(tr_remove);
+	_alpm_trans_free(tr_upgrade);
 	return(ret);
 }
 
-pmsyncpkg_t *_alpm_sync_find(alpm_list_t *syncpkgs, const char* pkgname)
-{
-	alpm_list_t *i;
-	for(i = syncpkgs; i; i = i->next) {
-		pmsyncpkg_t *syncpkg = i->data;
-		if(!syncpkg) {
-			continue;
-		}
-
-		pmpkg_t *pkg = alpm_sync_get_pkg(syncpkg);
-		if(strcmp(alpm_pkg_get_name(pkg), pkgname) == 0) {
-			return(syncpkg);
-		}
-	}
-
-	return(NULL); /* not found */
-}
-
-pmpkg_t SYMEXPORT *alpm_sync_get_pkg(const pmsyncpkg_t *sync)
-{
-	/* Sanity checks */
-	ASSERT(sync != NULL, return(NULL));
-
-	return sync->pkg;
-}
-
-alpm_list_t SYMEXPORT *alpm_sync_get_removes(const pmsyncpkg_t *sync)
-{
-	/* Sanity checks */
-	ASSERT(sync != NULL, return(NULL));
-
-	return sync->removes;
-}
-
 /* vim: set ts=2 sw=2 noet: */
