@@ -1,7 +1,8 @@
 /*
  *  sync.h
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005, 2006 by Miklos Vajna <vmiklos@frugalware.org>
  *
@@ -23,26 +24,12 @@
 
 #include "alpm.h"
 
-/* Sync package */
-struct __pmsyncpkg_t {
-	pmpkgreason_t newreason;
-	pmpkg_t *pkg;
-	alpm_list_t *removes;
-};
-
-pmsyncpkg_t *_alpm_sync_new(pmpkgreason_t newreason, pmpkg_t *spkg, alpm_list_t *removes);
-void _alpm_sync_free(pmsyncpkg_t *data);
-
-int _alpm_sync_sysupgrade(pmtrans_t *trans,
-		pmdb_t *db_local, alpm_list_t *dbs_sync, alpm_list_t **syncpkgs);
+int _alpm_sync_sysupgrade(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync, int enable_downgrade);
 
 int _alpm_sync_addtarget(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync, char *name);
 int _alpm_sync_prepare(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t *dbs_sync, alpm_list_t **data);
 int _alpm_sync_commit(pmtrans_t *trans, pmdb_t *db_local, alpm_list_t **data);
 
-/* typically trans->packages */
-pmsyncpkg_t *_alpm_sync_find(alpm_list_t *syncpkgs, const char* pkgname);
-
 #endif /* _ALPM_SYNC_H */
 
 /* vim: set ts=2 sw=2 noet: */
