@@ -1,7 +1,8 @@
 /*
  *  trans.c
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
  *  Copyright (c) 2005, 2006 by Miklos Vajna <vmiklos@frugalware.org>
@@ -26,7 +27,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#include <sys/wait.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/statvfs.h>
@@ -71,9 +71,11 @@ int SYMEXPORT alpm_trans_init(pmtranstype_t type, pmtransflag_t flags,
 	ASSERT(handle->trans == NULL, RET_ERR(PM_ERR_TRANS_NOT_NULL, -1));
 
 	/* lock db */
-	handle->lckfd = _alpm_lckmk();
-	if(handle->lckfd == -1) {
-		RET_ERR(PM_ERR_HANDLE_LOCK, -1);
+	if(!(flags & PM_TRANS_FLAG_NOLOCK)) {
+		handle->lckfd = _alpm_lckmk();
+		if(handle->lckfd == -1) {
+			RET_ERR(PM_ERR_HANDLE_LOCK, -1);
+		}
 	}
 
 	handle->trans = _alpm_trans_new();
@@ -87,7 +89,7 @@ int SYMEXPORT alpm_trans_init(pmtranstype_t type, pmtransflag_t flags,
 /** Search for packages to upgrade and add them to the transaction.
  * @return 0 on success, -1 on error (pm_errno is set accordingly)
  */
-int SYMEXPORT alpm_trans_sysupgrade()
+int SYMEXPORT alpm_trans_sysupgrade(int enable_downgrade)
 {
 	pmtrans_t *trans;
 
@@ -100,7 +102,7 @@ int SYMEXPORT alpm_trans_sysupgrade()
 	ASSERT(trans->state == STATE_INITIALIZED, RET_ERR(PM_ERR_TRANS_NOT_INITIALIZED, -1));
 	ASSERT(trans->type == PM_TRANS_TYPE_SYNC, RET_ERR(PM_ERR_TRANS_TYPE, -1));
 
-	return(_alpm_trans_sysupgrade(trans));
+	return(_alpm_sync_sysupgrade(trans, handle->db_local, handle->dbs_sync, enable_downgrade));
 }
 
 /** Add a target to the transaction.
@@ -158,6 +160,8 @@ int SYMEXPORT alpm_trans_commit(alpm_list_t **data)
 	ASSERT(handle->trans != NULL, RET_ERR(PM_ERR_TRANS_NULL, -1));
 	ASSERT(handle->trans->state == STATE_PREPARED, RET_ERR(PM_ERR_TRANS_NOT_PREPARED, -1));
 
+	ASSERT(!(handle->trans->flags & PM_TRANS_FLAG_NOLOCK), RET_ERR(PM_ERR_TRANS_NOT_LOCKED, -1));
+
 	return(_alpm_trans_commit(handle->trans, data));
 }
 
@@ -199,19 +203,23 @@ int SYMEXPORT alpm_trans_release()
 	ASSERT(trans != NULL, RET_ERR(PM_ERR_TRANS_NULL, -1));
 	ASSERT(trans->state != STATE_IDLE, RET_ERR(PM_ERR_TRANS_NULL, -1));
 
+	unsigned int nolock_flag = trans->flags & PM_TRANS_FLAG_NOLOCK;
+
 	_alpm_trans_free(trans);
 	handle->trans = NULL;
 
 	/* unlock db */
-	if(handle->lckfd != -1) {
-		while(close(handle->lckfd) == -1 && errno == EINTR);
-		handle->lckfd = -1;
-	}
-	if(_alpm_lckrm()) {
-		_alpm_log(PM_LOG_WARNING, _("could not remove lock file %s\n"),
-				alpm_option_get_lockfile());
-		alpm_logaction("warning: could not remove lock file %s\n",
-				alpm_option_get_lockfile());
+	if(!nolock_flag) {
+		if(handle->lckfd != -1) {
+			while(close(handle->lckfd) == -1 && errno == EINTR);
+			handle->lckfd = -1;
+		}
+		if(_alpm_lckrm()) {
+			_alpm_log(PM_LOG_WARNING, _("could not remove lock file %s\n"),
+					alpm_option_get_lockfile());
+			alpm_logaction("warning: could not remove lock file %s\n",
+					alpm_option_get_lockfile());
+		}
 	}
 
 	return(0);
@@ -226,15 +234,6 @@ pmtrans_t *_alpm_trans_new()
 	ALPM_LOG_FUNC;
 
 	CALLOC(trans, 1, sizeof(pmtrans_t), RET_ERR(PM_ERR_MEMORY, NULL));
-
-	trans->packages = NULL;
-	trans->skip_add = NULL;
-	trans->skip_remove = NULL;
-	trans->type = 0;
-	trans->flags = 0;
-	trans->cb_event = NULL;
-	trans->cb_conv = NULL;
-	trans->cb_progress = NULL;
 	trans->state = STATE_IDLE;
 
 	return(trans);
@@ -249,7 +248,7 @@ void _alpm_trans_free(pmtrans_t *trans)
 	}
 
 	if(trans->type == PM_TRANS_TYPE_SYNC) {
-		alpm_list_free_inner(trans->packages, (alpm_list_fn_free)_alpm_sync_free);
+		alpm_list_free_inner(trans->packages, (alpm_list_fn_free)_alpm_pkg_free_trans);
 	} else {
 		alpm_list_free_inner(trans->packages, (alpm_list_fn_free)_alpm_pkg_free);
 	}
@@ -280,17 +279,6 @@ int _alpm_trans_init(pmtrans_t *trans, pmtranstype_t type, pmtransflag_t flags,
 	return(0);
 }
 
-int _alpm_trans_sysupgrade(pmtrans_t *trans)
-{
-	ALPM_LOG_FUNC;
-
-	/* Sanity checks */
-	ASSERT(trans != NULL, RET_ERR(PM_ERR_TRANS_NULL, -1));
-
-	return(_alpm_sync_sysupgrade(trans, handle->db_local, handle->dbs_sync,
-				&(trans->packages)));
-}
-
 /** Add a target to the transaction.
  * @param trans the current transaction
  * @param target the name of the target to add
@@ -448,11 +436,8 @@ int _alpm_runscriptlet(const char *root, const char *installfn,
 	char scriptfn[PATH_MAX];
 	char cmdline[PATH_MAX];
 	char tmpdir[PATH_MAX];
-	char cwd[PATH_MAX];
 	char *scriptpath;
-	pid_t pid;
 	int clean_tmpdir = 0;
-	int restore_cwd = 0;
 	int retval = 0;
 
 	ALPM_LOG_FUNC;
@@ -463,13 +448,6 @@ int _alpm_runscriptlet(const char *root, const char *installfn,
 		return(0);
 	}
 
-	/* NOTE: popen will use the PARENT's /bin/sh, not the chroot's */
-	if(access("/bin/sh", X_OK)) {
-		/* not found */
-		_alpm_log(PM_LOG_ERROR, _("No /bin/sh in parent environment, aborting scriptlet\n"));
-		return(0);
-	}
-
 	/* creates a directory in $root/tmp/ for copying/extracting the scriptlet */
 	snprintf(tmpdir, PATH_MAX, "%stmp/", root);
 	if(access(tmpdir, F_OK) != 0) {
@@ -507,21 +485,6 @@ int _alpm_runscriptlet(const char *root, const char *installfn,
 		goto cleanup;
 	}
 
-	/* save the cwd so we can restore it later */
-	if(getcwd(cwd, PATH_MAX) == NULL) {
-		_alpm_log(PM_LOG_ERROR, _("could not get current working directory\n"));
-	} else {
-		restore_cwd = 1;
-	}
-
-	/* just in case our cwd was removed in the upgrade operation */
-	if(chdir(root) != 0) {
-		_alpm_log(PM_LOG_ERROR, _("could not change directory to %s (%s)\n"), root, strerror(errno));
-		goto cleanup;
-	}
-
-	_alpm_log(PM_LOG_DEBUG, "executing %s script...\n", script);
-
 	if(oldver) {
 		snprintf(cmdline, PATH_MAX, ". %s; %s %s %s",
 				scriptpath, script, ver, oldver);
@@ -529,77 +492,13 @@ int _alpm_runscriptlet(const char *root, const char *installfn,
 		snprintf(cmdline, PATH_MAX, ". %s; %s %s",
 				scriptpath, script, ver);
 	}
-	_alpm_log(PM_LOG_DEBUG, "%s\n", cmdline);
 
-	/* fork- parent and child each have seperate code blocks below */
-	pid = fork();
-	if(pid == -1) {
-		_alpm_log(PM_LOG_ERROR, _("could not fork a new process (%s)\n"), strerror(errno));
-		retval = 1;
-		goto cleanup;
-	}
-
-	if(pid == 0) {
-		FILE *pipe;
-		/* this code runs for the child only (the actual chroot/exec) */
-		_alpm_log(PM_LOG_DEBUG, "chrooting in %s\n", root);
-		if(chroot(root) != 0) {
-			_alpm_log(PM_LOG_ERROR, _("could not change the root directory (%s)\n"),
-					strerror(errno));
-			exit(1);
-		}
-		if(chdir("/") != 0) {
-			_alpm_log(PM_LOG_ERROR, _("could not change directory to / (%s)\n"),
-					strerror(errno));
-			exit(1);
-		}
-		umask(0022);
-		_alpm_log(PM_LOG_DEBUG, "executing \"%s\"\n", cmdline);
-		/* execl("/bin/sh", "sh", "-c", cmdline, (char *)NULL); */
-		pipe = popen(cmdline, "r");
-		if(!pipe) {
-			_alpm_log(PM_LOG_ERROR, _("call to popen failed (%s)"),
-					strerror(errno));
-			exit(1);
-		}
-		while(!feof(pipe)) {
-			char line[PATH_MAX];
-			if(fgets(line, PATH_MAX, pipe) == NULL)
-				break;
-			alpm_logaction("%s", line);
-			EVENT(trans, PM_TRANS_EVT_SCRIPTLET_INFO, line, NULL);
-		}
-		retval = pclose(pipe);
-		exit(WEXITSTATUS(retval));
-	} else {
-		/* this code runs for the parent only (wait on the child) */
-		pid_t retpid;
-		int status;
-		while((retpid = waitpid(pid, &status, 0)) == -1 && errno == EINTR);
-		if(retpid == -1) {
-			_alpm_log(PM_LOG_ERROR, _("call to waitpid failed (%s)\n"),
-			          strerror(errno));
-			retval = 1;
-			goto cleanup;
-		} else {
-			/* check the return status, make sure it is 0 (success) */
-			if(WIFEXITED(status)) {
-				_alpm_log(PM_LOG_DEBUG, "call to waitpid succeeded\n");
-				if(WEXITSTATUS(status) != 0) {
-					_alpm_log(PM_LOG_ERROR, _("scriptlet failed to execute correctly\n"));
-					retval = 1;
-				}
-			}
-		}
-	}
+	retval = _alpm_run_chroot(root, cmdline);
 
 cleanup:
 	if(clean_tmpdir && _alpm_rmrf(tmpdir)) {
 		_alpm_log(PM_LOG_WARNING, _("could not remove tmpdir %s\n"), tmpdir);
 	}
-	if(restore_cwd) {
-		chdir(cwd);
-	}
 
 	return(retval);
 }
