@@ -1,7 +1,8 @@
 /*
  *  conf.c
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -44,10 +45,6 @@ config_t *config_new(void)
 	newconfig->logmask = PM_LOG_ERROR | PM_LOG_WARNING;
 	/* CONFFILE is defined at compile-time */
 	newconfig->configfile = strdup(CONFFILE);
-	newconfig->rootdir = NULL;
-	newconfig->dbpath = NULL;
-	newconfig->logfile = NULL;
-	newconfig->syncfirst = NULL;
 
 	return(newconfig);
 }
@@ -58,11 +55,13 @@ int config_free(config_t *oldconfig)
 		return(-1);
 	}
 
+	FREELIST(oldconfig->holdpkg);
 	FREELIST(oldconfig->syncfirst);
 	free(oldconfig->configfile);
 	free(oldconfig->rootdir);
 	free(oldconfig->dbpath);
 	free(oldconfig->logfile);
+	free(oldconfig->xfercommand);
 	free(oldconfig);
 	oldconfig = NULL;
 
