@@ -172,7 +172,7 @@ static int query_search(alpm_list_t *targets)
 		}
 
 		/* print the package size with the output if ShowSize option set */
-		if(config->showsize) {
+		if(!config->quiet && config->showsize) {
 			/* Convert byte size to MB */
 			double mbsize = (double)alpm_pkg_get_size(pkg) / (1024.0 * 1024.0);
 
@@ -344,7 +344,7 @@ static int check(pmpkg_t *pkg)
 			if(config->quiet) {
 				printf("%s %s\n", pkgname, f);
 			} else {
-				pm_printf(PM_LOG_WARNING, "%s: missing %s (%s)\n",
+				pm_printf(PM_LOG_WARNING, "%s: %s (%s)\n",
 						pkgname, f, strerror(errno));
 			}
 			errors++;
@@ -352,7 +352,7 @@ static int check(pmpkg_t *pkg)
 	}
 
 	if(!config->quiet) {
-		printf("%s: %d total, %d missing file(s)\n",
+		printf(_("%s: %d total files, %d missing file(s)\n"),
 				pkgname, allfiles, errors);
 	}
 
@@ -394,6 +394,7 @@ static int display(pmpkg_t *pkg)
 int pacman_query(alpm_list_t *targets)
 {
 	int ret = 0;
+	int match = 0;
 	alpm_list_t *i;
 	pmpkg_t *pkg = NULL;
 
@@ -436,8 +437,12 @@ int pacman_query(alpm_list_t *targets)
 				if(value != 0) {
 					ret = 1;
 				}
+				match = 1;
 			}
 		}
+		if(!match) {
+			ret = 1;
+		}
 		return(ret);
 	}
 
@@ -471,6 +476,7 @@ int pacman_query(alpm_list_t *targets)
 			if(value != 0) {
 				ret = 1;
 			}
+			match = 1;
 		}
 
 		if(config->op_q_isfile) {
@@ -479,6 +485,10 @@ int pacman_query(alpm_list_t *targets)
 		}
 	}
 
+	if(!match) {
+		ret = 1;
+	}
+
 	return(ret);
 }
 
